@@ -1,5 +1,6 @@
 // Copyright 2018-2022 the Deno authors. All rights reserved. MIT license.
 
+use deno_core::error::bad_resource_id;
 use deno_core::error::not_supported;
 use deno_core::error::resource_unavailable;
 use deno_core::error::AnyError;
@@ -70,13 +71,67 @@ pub fn init() -> Extension {
         .build()
 }
 
-pub fn init_stdio() -> Extension {
+pub enum StdioPipe {
+    Inherit,
+    File(StdFile),
+}
+
+impl Default for StdioPipe {
+    fn default() -> Self {
+        Self::Inherit
+    }
+}
+
+impl Clone for StdioPipe {
+    fn clone(&self) -> Self {
+        match self {
+            StdioPipe::Inherit => StdioPipe::Inherit,
+            StdioPipe::File(pipe) => StdioPipe::File(pipe.try_clone().unwrap()),
+        }
+    }
+}
+
+/// Specify how stdin, stdout, and stderr are piped.
+/// By default, inherits from the process.
+#[derive(Clone, Default)]
+pub struct Stdio {
+    pub stdin: StdioPipe,
+    pub stdout: StdioPipe,
+    pub stderr: StdioPipe,
+}
+
+pub fn init_stdio(stdio: Stdio) -> Extension {
+    // todo(dsheret): don't do this? Taking out the writers was necessary to prevent invalid handle panics
+    let stdio = Rc::new(RefCell::new(Some(stdio)));
+
     Extension::builder()
-        .state(|state| {
+        .state(move |state| {
+            let stdio = stdio
+                .borrow_mut()
+                .take()
+                .expect("Extension only supports being used once.");
             let t = &mut state.resource_table;
-            t.add(StdFileResource::stdio(&STDIN_HANDLE, "stdin"));
-            t.add(StdFileResource::stdio(&STDOUT_HANDLE, "stdout"));
-            t.add(StdFileResource::stdio(&STDERR_HANDLE, "stderr"));
+            t.add(StdFileResource::stdio(
+                match &stdio.stdin {
+                    StdioPipe::Inherit => &STDIN_HANDLE,
+                    StdioPipe::File(pipe) => pipe,
+                },
+                "stdin",
+            ));
+            t.add(StdFileResource::stdio(
+                match &stdio.stdout {
+                    StdioPipe::Inherit => &STDOUT_HANDLE,
+                    StdioPipe::File(pipe) => pipe,
+                },
+                "stdout",
+            ));
+            t.add(StdFileResource::stdio(
+                match &stdio.stderr {
+                    StdioPipe::Inherit => &STDERR_HANDLE,
+                    StdioPipe::File(pipe) => pipe,
+                },
+                "stderr",
+            ));
             Ok(())
         })
         .build()
@@ -306,6 +361,13 @@ impl StdFileResource {
             None => Err(resource_unavailable()),
         }
     }
+
+    pub fn clone_file(state: &mut OpState, rid: ResourceId) -> Result<std::fs::File, AnyError> {
+        Self::with(state, rid, move |r| match r {
+            Ok(std_file) => std_file.try_clone().map_err(AnyError::from),
+            Err(_) => Err(bad_resource_id()),
+        })
+    }
 }
 
 impl Resource for StdFileResource {
