@@ -3,10 +3,12 @@
 use crate::inspector_server::InspectorServer;
 use crate::js;
 use crate::ops;
+use crate::ops::io::Stdio;
 use crate::permissions::Permissions;
 use crate::BootstrapOptions;
 use deno_broadcast_channel::InMemoryBroadcastChannel;
 use deno_core::error::AnyError;
+use deno_core::error::JsError;
 use deno_core::futures::Future;
 use deno_core::located_script_name;
 use deno_core::resolve_url_or_path;
@@ -36,6 +38,8 @@ use std::task::Poll;
 use crate::StartSnapshot;
 use derive_builder::Builder;
 
+pub type FormatJsErrorFn = dyn Fn(&JsError) -> String + Sync + Send;
+
 /// This worker is created and used by almost all
 /// subcommands in Deno executable.
 ///
@@ -65,6 +69,7 @@ pub struct WorkerOptions {
     // Callbacks invoked when creating new instance of WebWorker
     pub create_web_worker_cb: Arc<ops::worker_host::CreateWebWorkerCb>,
     pub web_worker_preload_module_cb: Arc<ops::worker_host::PreloadModuleCb>,
+    pub format_js_error_fn: Option<Arc<FormatJsErrorFn>>,
     // pub source_map_getter: Option<Box<dyn SourceMapGetter>>,
     pub js_error_create_fn: Option<Rc<JsErrorCreateFn>>,
     pub maybe_inspector_server: Option<Arc<InspectorServer>>,
@@ -75,6 +80,7 @@ pub struct WorkerOptions {
     pub broadcast_channel: InMemoryBroadcastChannel,
     pub shared_array_buffer_store: Option<SharedArrayBufferStore>,
     pub compiled_wasm_module_store: Option<CompiledWasmModuleStore>,
+    pub stdio: Stdio,
 
     #[builder(setter(custom))]
     pub main_module: Option<ModuleSpecifier>,
@@ -148,12 +154,13 @@ impl MainWorker {
             ops::worker_host::init(
                 options.create_web_worker_cb.clone(),
                 options.web_worker_preload_module_cb.clone(),
+                options.format_js_error_fn.clone(),
             ),
             ops::spawn::init(),
             ops::fs_events::init(),
             ops::fs::init(),
             ops::io::init(),
-            ops::io::init_stdio(),
+            ops::io::init_stdio(options.stdio),
             deno_tls::init(),
             deno_net::init::<Permissions>(
                 options.root_cert_store.clone(),
@@ -187,7 +194,6 @@ impl MainWorker {
             module_loader: Some(options.module_loader.clone()),
             startup_snapshot: Some(snapshot),
             // source_map_getter: options.source_map_getter,
-            js_error_create_fn: options.js_error_create_fn.clone(),
             get_error_class_fn: options.get_error_class_fn,
             shared_array_buffer_store: options.shared_array_buffer_store.clone(),
             compiled_wasm_module_store: options.compiled_wasm_module_store.clone(),
